@@ -2,94 +2,67 @@
 from __future__ import annotations
 
 import asyncio
+from collections.abc import Awaitable, Callable, Coroutine
 from functools import wraps
 import logging
+from typing import Any, TypeVar
 
 import aiohttp.client_exceptions
-from iaqualink import (
+from iaqualink.client import AqualinkClient
+from iaqualink.device import (
     AqualinkBinarySensor,
-    AqualinkClient,
     AqualinkDevice,
     AqualinkLight,
-    AqualinkLoginException,
     AqualinkSensor,
     AqualinkThermostat,
     AqualinkToggle,
 )
-import voluptuous as vol
+from iaqualink.exception import AqualinkServiceException
+from typing_extensions import Concatenate, ParamSpec
 
-from homeassistant import config_entries
 from homeassistant.components.binary_sensor import DOMAIN as BINARY_SENSOR_DOMAIN
 from homeassistant.components.climate import DOMAIN as CLIMATE_DOMAIN
 from homeassistant.components.light import DOMAIN as LIGHT_DOMAIN
 from homeassistant.components.sensor import DOMAIN as SENSOR_DOMAIN
 from homeassistant.components.switch import DOMAIN as SWITCH_DOMAIN
 from homeassistant.config_entries import ConfigEntry
-from homeassistant.const import CONF_PASSWORD, CONF_USERNAME
+from homeassistant.const import CONF_PASSWORD, CONF_USERNAME, Platform
 from homeassistant.core import HomeAssistant
 from homeassistant.exceptions import ConfigEntryNotReady
 from homeassistant.helpers.aiohttp_client import async_get_clientsession
-import homeassistant.helpers.config_validation as cv
 from homeassistant.helpers.dispatcher import (
     async_dispatcher_connect,
     async_dispatcher_send,
 )
 from homeassistant.helpers.entity import DeviceInfo, Entity
 from homeassistant.helpers.event import async_track_time_interval
-from homeassistant.helpers.typing import ConfigType
 
 from .const import DOMAIN, UPDATE_INTERVAL
 
+_AqualinkEntityT = TypeVar("_AqualinkEntityT", bound="AqualinkEntity")
+_P = ParamSpec("_P")
+
 _LOGGER = logging.getLogger(__name__)
 
 ATTR_CONFIG = "config"
 PARALLEL_UPDATES = 0
 
 PLATFORMS = [
-    BINARY_SENSOR_DOMAIN,
-    CLIMATE_DOMAIN,
-    LIGHT_DOMAIN,
-    SENSOR_DOMAIN,
-    SWITCH_DOMAIN,
+    Platform.BINARY_SENSOR,
+    Platform.CLIMATE,
+    Platform.LIGHT,
+    Platform.SENSOR,
+    Platform.SWITCH,
 ]
 
-CONFIG_SCHEMA = vol.Schema(
-    vol.All(
-        cv.deprecated(DOMAIN),
-        {
-            DOMAIN: vol.Schema(
-                {
-                    vol.Required(CONF_USERNAME): cv.string,
-                    vol.Required(CONF_PASSWORD): cv.string,
-                }
-            )
-        },
-    ),
-    extra=vol.ALLOW_EXTRA,
-)
 
-
-async def async_setup(hass: HomeAssistant, config: ConfigType) -> bool:
-    """Set up the Aqualink component."""
-    conf = config.get(DOMAIN)
-
-    hass.data[DOMAIN] = {}
-
-    if conf is not None:
-        hass.async_create_task(
-            hass.config_entries.flow.async_init(
-                DOMAIN, context={"source": config_entries.SOURCE_IMPORT}, data=conf
-            )
-        )
-
-    return True
-
-
-async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> None:
+async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
     """Set up Aqualink from a config entry."""
     username = entry.data[CONF_USERNAME]
     password = entry.data[CONF_PASSWORD]
 
+    hass.data.setdefault(DOMAIN, {})
+
     # These will contain the initialized devices
     binary_sensors = hass.data[DOMAIN][BINARY_SENSOR_DOMAIN] = []
     climates = hass.data[DOMAIN][CLIMATE_DOMAIN] = []
@@ -101,24 +74,36 @@ async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> None:
     aqualink = AqualinkClient(username, password, session)
     try:
         await aqualink.login()
-    except AqualinkLoginException as login_exception:
+    except AqualinkServiceException as login_exception:
         _LOGGER.error("Failed to login: %s", login_exception)
         return False
     except (
         asyncio.TimeoutError,
         aiohttp.client_exceptions.ClientConnectorError,
     ) as aio_exception:
-        _LOGGER.warning("Exception raised while attempting to login: %s", aio_exception)
-        raise ConfigEntryNotReady from aio_exception
+        raise ConfigEntryNotReady(
+            f"Error while attempting login: {aio_exception}"
+        ) from aio_exception
+
+    try:
+        systems = await aqualink.get_systems()
+    except AqualinkServiceException as svc_exception:
+        raise ConfigEntryNotReady(
+            f"Error while attempting to retrieve systems list: {svc_exception}"
+        ) from svc_exception
 
-    systems = await aqualink.get_systems()
     systems = list(systems.values())
     if not systems:
         _LOGGER.error("No systems detected or supported")
         return False
 
     # Only supporting the first system for now.
-    devices = await systems[0].get_devices()
+    try:
+        devices = await systems[0].get_devices()
+    except AqualinkServiceException as svc_exception:
+        raise ConfigEntryNotReady(
+            f"Error while attempting to retrieve devices list: {svc_exception}"
+        ) from svc_exception
 
     for dev in devices.values():
         if isinstance(dev, AqualinkThermostat):
@@ -135,31 +120,33 @@ async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> None:
     forward_setup = hass.config_entries.async_forward_entry_setup
     if binary_sensors:
         _LOGGER.debug("Got %s binary sensors: %s", len(binary_sensors), binary_sensors)
-        hass.async_create_task(forward_setup(entry, BINARY_SENSOR_DOMAIN))
+        hass.async_create_task(forward_setup(entry, Platform.BINARY_SENSOR))
     if climates:
         _LOGGER.debug("Got %s climates: %s", len(climates), climates)
-        hass.async_create_task(forward_setup(entry, CLIMATE_DOMAIN))
+        hass.async_create_task(forward_setup(entry, Platform.CLIMATE))
     if lights:
         _LOGGER.debug("Got %s lights: %s", len(lights), lights)
-        hass.async_create_task(forward_setup(entry, LIGHT_DOMAIN))
+        hass.async_create_task(forward_setup(entry, Platform.LIGHT))
     if sensors:
         _LOGGER.debug("Got %s sensors: %s", len(sensors), sensors)
-        hass.async_create_task(forward_setup(entry, SENSOR_DOMAIN))
+        hass.async_create_task(forward_setup(entry, Platform.SENSOR))
     if switches:
         _LOGGER.debug("Got %s switches: %s", len(switches), switches)
-        hass.async_create_task(forward_setup(entry, SWITCH_DOMAIN))
+        hass.async_create_task(forward_setup(entry, Platform.SWITCH))
 
     async def _async_systems_update(now):
         """Refresh internal state for all systems."""
-        prev = systems[0].last_run_success
+        prev = systems[0].online
 
-        await systems[0].update()
-        success = systems[0].last_run_success
-
-        if not success and prev:
-            _LOGGER.warning("Failed to refresh iAqualink state")
-        elif success and not prev:
-            _LOGGER.warning("Reconnected to iAqualink")
+        try:
+            await systems[0].update()
+        except AqualinkServiceException as svc_exception:
+            if prev is not None:
+                _LOGGER.warning("Failed to refresh iAqualink state: %s", svc_exception)
+        else:
+            cur = systems[0].online
+            if cur is True and prev is not True:
+                _LOGGER.warning("Reconnected to iAqualink")
 
         async_dispatcher_send(hass, DOMAIN)
 
@@ -174,16 +161,20 @@ async def async_unload_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
         platform for platform in PLATFORMS if platform in hass.data[DOMAIN]
     ]
 
-    hass.data[DOMAIN].clear()
+    del hass.data[DOMAIN]
 
     return await hass.config_entries.async_unload_platforms(entry, platforms_to_unload)
 
 
-def refresh_system(func):
+def refresh_system(
+    func: Callable[Concatenate[_AqualinkEntityT, _P], Awaitable[Any]]
+) -> Callable[Concatenate[_AqualinkEntityT, _P], Coroutine[Any, Any, None]]:
     """Force update all entities after state change."""
 
     @wraps(func)
-    async def wrapper(self, *args, **kwargs):
+    async def wrapper(
+        self: _AqualinkEntityT, *args: _P.args, **kwargs: _P.kwargs
+    ) -> None:
         """Call decorated function and send update signal to all entities."""
         await func(self, *args, **kwargs)
         async_dispatcher_send(self.hass, DOMAIN)
@@ -228,20 +219,20 @@ class AqualinkEntity(Entity):
     @property
     def assumed_state(self) -> bool:
         """Return whether the state is based on actual reading from the device."""
-        return not self.dev.system.last_run_success
+        return self.dev.system.online in [False, None]
 
     @property
     def available(self) -> bool:
         """Return whether the device is available or not."""
-        return self.dev.system.online
+        return self.dev.system.online is True
 
     @property
     def device_info(self) -> DeviceInfo:
         """Return the device info."""
-        return {
-            "identifiers": {(DOMAIN, self.unique_id)},
-            "name": self.name,
-            "model": self.dev.__class__.__name__.replace("Aqualink", ""),
-            "manufacturer": "Jandy",
-            "via_device": (DOMAIN, self.dev.system.serial),
-        }
+        return DeviceInfo(
+            identifiers={(DOMAIN, self.unique_id)},
+            manufacturer="Jandy",
+            model=self.dev.__class__.__name__.replace("Aqualink", ""),
+            name=self.name,
+            via_device=(DOMAIN, self.dev.system.serial),
+        )
