@@ -1,30 +1,28 @@
 """Component to integrate ambilight for TVs exposing the Joint Space API."""
 from __future__ import annotations
 
-from typing import Any
+from dataclasses import dataclass
 
 from haphilipsjs import PhilipsTV
 from haphilipsjs.typing import AmbilightCurrentConfiguration
 
-from homeassistant import config_entries
 from homeassistant.components.light import (
     ATTR_BRIGHTNESS,
     ATTR_EFFECT,
     ATTR_HS_COLOR,
-    COLOR_MODE_HS,
-    COLOR_MODE_ONOFF,
-    SUPPORT_BRIGHTNESS,
-    SUPPORT_COLOR,
-    SUPPORT_EFFECT,
+    ColorMode,
     LightEntity,
+    LightEntityFeature,
 )
+from homeassistant.config_entries import ConfigEntry
 from homeassistant.core import HomeAssistant, callback
 from homeassistant.helpers.entity import DeviceInfo
+from homeassistant.helpers.entity_platform import AddEntitiesCallback
 from homeassistant.helpers.update_coordinator import CoordinatorEntity
 from homeassistant.util.color import color_hsv_to_RGB, color_RGB_to_hsv
 
 from . import PhilipsTVDataUpdateCoordinator
-from .const import CONF_SYSTEM, DOMAIN
+from .const import DOMAIN
 
 EFFECT_PARTITION = ": "
 EFFECT_MODE = "Mode"
@@ -35,18 +33,12 @@ EFFECT_EXPERT_STYLES = {"FOLLOW_AUDIO", "FOLLOW_COLOR", "Lounge light"}
 
 async def async_setup_entry(
     hass: HomeAssistant,
-    config_entry: config_entries.ConfigEntry,
-    async_add_entities,
-):
+    config_entry: ConfigEntry,
+    async_add_entities: AddEntitiesCallback,
+) -> None:
     """Set up the configuration entry."""
     coordinator = hass.data[DOMAIN][config_entry.entry_id]
-    async_add_entities(
-        [
-            PhilipsTVLightEntity(
-                coordinator, config_entry.data[CONF_SYSTEM], config_entry.unique_id
-            )
-        ]
-    )
+    async_add_entities([PhilipsTVLightEntity(coordinator)])
 
 
 def _get_settings(style: AmbilightCurrentConfiguration):
@@ -58,44 +50,54 @@ def _get_settings(style: AmbilightCurrentConfiguration):
     return None
 
 
-def _parse_effect(effect: str):
-    style, _, algorithm = effect.partition(EFFECT_PARTITION)
-    if style == EFFECT_MODE:
-        return EFFECT_MODE, algorithm, None
-    algorithm, _, expert = algorithm.partition(EFFECT_PARTITION)
-    if expert:
-        return EFFECT_EXPERT, style, algorithm
-    return EFFECT_AUTO, style, algorithm
+@dataclass
+class AmbilightEffect:
+    """Data class describing the ambilight effect."""
 
+    mode: str
+    style: str
+    algorithm: str | None = None
 
-def _get_effect(mode: str, style: str, algorithm: str | None):
-    if mode == EFFECT_MODE:
-        return f"{EFFECT_MODE}{EFFECT_PARTITION}{style}"
-    if mode == EFFECT_EXPERT:
-        return f"{style}{EFFECT_PARTITION}{algorithm}{EFFECT_PARTITION}{EFFECT_EXPERT}"
-    return f"{style}{EFFECT_PARTITION}{algorithm}"
+    def is_on(self, powerstate) -> bool:
+        """Check whether the ambilight is considered on."""
+        if self.mode in (EFFECT_AUTO, EFFECT_EXPERT):
+            if self.style in ("FOLLOW_VIDEO", "FOLLOW_AUDIO"):
+                return powerstate in ("On", None)
+            if self.style == "OFF":
+                return False
+            return True
 
+        if self.mode == EFFECT_MODE:
+            if self.style == "internal":
+                return powerstate in ("On", None)
+            return True
 
-def _is_on(mode, style, powerstate):
-    if mode in (EFFECT_AUTO, EFFECT_EXPERT):
-        if style in ("FOLLOW_VIDEO", "FOLLOW_AUDIO"):
-            return powerstate in ("On", None)
-        if style == "OFF":
-            return False
+        return False
+
+    def is_valid(self) -> bool:
+        """Validate the effect configuration."""
+        if self.mode == EFFECT_EXPERT:
+            return self.style in EFFECT_EXPERT_STYLES
         return True
 
-    if mode == EFFECT_MODE:
-        if style == "internal":
-            return powerstate in ("On", None)
-        return True
+    @staticmethod
+    def from_str(effect_string: str) -> AmbilightEffect:
+        """Create AmbilightEffect object from string."""
+        style, _, algorithm = effect_string.partition(EFFECT_PARTITION)
+        if style == EFFECT_MODE:
+            return AmbilightEffect(mode=EFFECT_MODE, style=algorithm, algorithm=None)
+        algorithm, _, expert = algorithm.partition(EFFECT_PARTITION)
+        if expert:
+            return AmbilightEffect(mode=EFFECT_EXPERT, style=style, algorithm=algorithm)
+        return AmbilightEffect(mode=EFFECT_AUTO, style=style, algorithm=algorithm)
 
-    return False
-
-
-def _is_valid(mode, style):
-    if mode == EFFECT_EXPERT:
-        return style in EFFECT_EXPERT_STYLES
-    return True
+    def __str__(self) -> str:
+        """Get a string representation of the effect."""
+        if self.mode == EFFECT_MODE:
+            return f"{EFFECT_MODE}{EFFECT_PARTITION}{self.style}"
+        if self.mode == EFFECT_EXPERT:
+            return f"{self.style}{EFFECT_PARTITION}{self.algorithm}{EFFECT_PARTITION}{EFFECT_EXPERT}"
+        return f"{self.style}{EFFECT_PARTITION}{self.algorithm}"
 
 
 def _get_cache_keys(device: PhilipsTV):
@@ -130,106 +132,103 @@ def _average_pixels(data):
     return 0.0, 0.0, 0.0
 
 
-class PhilipsTVLightEntity(CoordinatorEntity, LightEntity):
+class PhilipsTVLightEntity(
+    CoordinatorEntity[PhilipsTVDataUpdateCoordinator], LightEntity
+):
     """Representation of a Philips TV exposing the JointSpace API."""
 
     def __init__(
         self,
         coordinator: PhilipsTVDataUpdateCoordinator,
-        system: dict[str, Any],
-        unique_id: str,
     ) -> None:
         """Initialize light."""
         self._tv = coordinator.api
         self._hs = None
         self._brightness = None
-        self._system = system
-        self._coordinator = coordinator
         self._cache_keys = None
+        self._last_selected_effect: AmbilightEffect = None
         super().__init__(coordinator)
 
-        self._attr_supported_color_modes = [COLOR_MODE_HS, COLOR_MODE_ONOFF]
-        self._attr_supported_features = (
-            SUPPORT_EFFECT | SUPPORT_COLOR | SUPPORT_BRIGHTNESS
-        )
-        self._attr_name = self._system["name"]
-        self._attr_unique_id = unique_id
+        self._attr_supported_color_modes = {ColorMode.HS, ColorMode.ONOFF}
+        self._attr_supported_features = LightEntityFeature.EFFECT
+        self._attr_name = f"{coordinator.system['name']} Ambilight"
+        self._attr_unique_id = coordinator.unique_id
         self._attr_icon = "mdi:television-ambient-light"
         self._attr_device_info = DeviceInfo(
             identifiers={
                 (DOMAIN, self._attr_unique_id),
             },
             manufacturer="Philips",
-            model=self._system.get("model"),
-            name=self._system["name"],
-            sw_version=self._system.get("softwareversion"),
+            model=coordinator.system.get("model"),
+            name=coordinator.system["name"],
+            sw_version=coordinator.system.get("softwareversion"),
         )
 
         self._update_from_coordinator()
 
     def _calculate_effect_list(self):
         """Calculate an effect list based on current status."""
-        effects = []
+        effects: list[AmbilightEffect] = []
         effects.extend(
-            _get_effect(EFFECT_AUTO, style, setting)
+            AmbilightEffect(mode=EFFECT_AUTO, style=style, algorithm=setting)
             for style, data in self._tv.ambilight_styles.items()
-            if _is_valid(EFFECT_AUTO, style)
-            and _is_on(EFFECT_AUTO, style, self._tv.powerstate)
             for setting in data.get("menuSettings", [])
         )
 
         effects.extend(
-            _get_effect(EFFECT_EXPERT, style, algorithm)
+            AmbilightEffect(mode=EFFECT_EXPERT, style=style, algorithm=algorithm)
             for style, data in self._tv.ambilight_styles.items()
-            if _is_valid(EFFECT_EXPERT, style)
-            and _is_on(EFFECT_EXPERT, style, self._tv.powerstate)
             for algorithm in data.get("algorithms", [])
         )
 
         effects.extend(
-            _get_effect(EFFECT_MODE, style, None)
+            AmbilightEffect(mode=EFFECT_MODE, style=style)
             for style in self._tv.ambilight_modes
-            if _is_valid(EFFECT_MODE, style)
-            and _is_on(EFFECT_MODE, style, self._tv.powerstate)
         )
 
-        return sorted(effects)
+        filtered_effects = [
+            str(effect)
+            for effect in effects
+            if effect.is_valid() and effect.is_on(self._tv.powerstate)
+        ]
 
-    def _calculate_effect(self):
+        return sorted(filtered_effects)
+
+    def _calculate_effect(self) -> AmbilightEffect:
         """Return the current effect."""
         current = self._tv.ambilight_current_configuration
         if current and self._tv.ambilight_mode != "manual":
             if current["isExpert"]:
                 if settings := _get_settings(current):
-                    return _get_effect(
+                    return AmbilightEffect(
                         EFFECT_EXPERT, current["styleName"], settings["algorithm"]
                     )
-                return _get_effect(EFFECT_EXPERT, current["styleName"], None)
+                return AmbilightEffect(EFFECT_EXPERT, current["styleName"], None)
 
-            return _get_effect(
+            return AmbilightEffect(
                 EFFECT_AUTO, current["styleName"], current.get("menuSetting", None)
             )
 
-        return _get_effect(EFFECT_MODE, self._tv.ambilight_mode, None)
+        return AmbilightEffect(EFFECT_MODE, self._tv.ambilight_mode, None)
 
     @property
-    def color_mode(self):
+    def color_mode(self) -> ColorMode:
         """Return the current color mode."""
         current = self._tv.ambilight_current_configuration
         if current and current["isExpert"]:
-            return COLOR_MODE_HS
+            return ColorMode.HS
 
         if self._tv.ambilight_mode in ["manual", "expert"]:
-            return COLOR_MODE_HS
+            return ColorMode.HS
 
-        return COLOR_MODE_ONOFF
+        return ColorMode.ONOFF
 
     @property
     def is_on(self):
         """Return if the light is turned on."""
         if self._tv.on:
-            mode, style, _ = _parse_effect(self.effect)
-            return _is_on(mode, style, self._tv.powerstate)
+            effect = AmbilightEffect.from_str(self.effect)
+            return effect.is_on(self._tv.powerstate)
 
         return False
 
@@ -240,21 +239,23 @@ class PhilipsTVLightEntity(CoordinatorEntity, LightEntity):
         if (cache_keys := _get_cache_keys(self._tv)) != self._cache_keys:
             self._cache_keys = cache_keys
             self._attr_effect_list = self._calculate_effect_list()
-            self._attr_effect = self._calculate_effect()
+            self._attr_effect = str(self._calculate_effect())
 
         if current and current["isExpert"]:
             if settings := _get_settings(current):
                 color = settings["color"]
 
-        mode, _, _ = _parse_effect(self._attr_effect)
+        effect = AmbilightEffect.from_str(self._attr_effect)
+        if effect.is_on(self._tv.powerstate):
+            self._last_selected_effect = effect
 
-        if mode == EFFECT_EXPERT and color:
+        if effect.mode == EFFECT_EXPERT and color:
             self._attr_hs_color = (
                 color["hue"] * 360.0 / 255.0,
                 color["saturation"] * 100.0 / 255.0,
             )
             self._attr_brightness = color["brightness"]
-        elif mode == EFFECT_MODE and self._tv.ambilight_cached:
+        elif effect.mode == EFFECT_MODE and self._tv.ambilight_cached:
             hsv_h, hsv_s, hsv_v = color_RGB_to_hsv(
                 *_average_pixels(self._tv.ambilight_cached)
             )
@@ -270,7 +271,9 @@ class PhilipsTVLightEntity(CoordinatorEntity, LightEntity):
         self._update_from_coordinator()
         super()._handle_coordinator_update()
 
-    async def _set_ambilight_cached(self, algorithm, hs_color, brightness):
+    async def _set_ambilight_cached(
+        self, effect: AmbilightEffect, hs_color: tuple[float, float], brightness: int
+    ):
         """Set ambilight via the manual or expert mode."""
         rgb = color_hsv_to_RGB(hs_color[0], hs_color[1], brightness * 100 / 255)
 
@@ -283,21 +286,21 @@ class PhilipsTVLightEntity(CoordinatorEntity, LightEntity):
         if not await self._tv.setAmbilightCached(data):
             raise Exception("Failed to set ambilight color")
 
-        if algorithm != self._tv.ambilight_mode:
-            if not await self._tv.setAmbilightMode(algorithm):
+        if effect.style != self._tv.ambilight_mode:
+            if not await self._tv.setAmbilightMode(effect.style):
                 raise Exception("Failed to set ambilight mode")
 
     async def _set_ambilight_expert_config(
-        self, style, algorithm, hs_color, brightness
+        self, effect: AmbilightEffect, hs_color: tuple[float, float], brightness: int
     ):
         """Set ambilight via current configuration."""
         config: AmbilightCurrentConfiguration = {
-            "styleName": style,
+            "styleName": effect.style,
             "isExpert": True,
         }
 
         setting = {
-            "algorithm": algorithm,
+            "algorithm": effect.algorithm,
             "color": {
                 "hue": round(hs_color[0] * 255.0 / 360.0),
                 "saturation": round(hs_color[1] * 255.0 / 100.0),
@@ -310,23 +313,23 @@ class PhilipsTVLightEntity(CoordinatorEntity, LightEntity):
             },
         }
 
-        if style in ("FOLLOW_COLOR", "Lounge light"):
+        if effect.style in ("FOLLOW_COLOR", "Lounge light"):
             config["colorSettings"] = setting
             config["speed"] = 2
 
-        elif style == "FOLLOW_AUDIO":
+        elif effect.style == "FOLLOW_AUDIO":
             config["audioSettings"] = setting
             config["tuning"] = 0
 
         if not await self._tv.setAmbilightCurrentConfiguration(config):
             raise Exception("Failed to set ambilight mode")
 
-    async def _set_ambilight_config(self, style, algorithm):
+    async def _set_ambilight_config(self, effect: AmbilightEffect):
         """Set ambilight via current configuration."""
         config: AmbilightCurrentConfiguration = {
-            "styleName": style,
+            "styleName": effect.style,
             "isExpert": False,
-            "menuSetting": algorithm,
+            "menuSetting": effect.algorithm,
         }
 
         if await self._tv.setAmbilightCurrentConfiguration(config) is False:
@@ -336,35 +339,39 @@ class PhilipsTVLightEntity(CoordinatorEntity, LightEntity):
         """Turn the bulb on."""
         brightness = kwargs.get(ATTR_BRIGHTNESS, self.brightness)
         hs_color = kwargs.get(ATTR_HS_COLOR, self.hs_color)
-        effect = kwargs.get(ATTR_EFFECT, self.effect)
+        attr_effect = kwargs.get(ATTR_EFFECT, self.effect)
 
         if not self._tv.on:
             raise Exception("TV is not available")
 
-        mode, style, setting = _parse_effect(effect)
+        effect = AmbilightEffect.from_str(attr_effect)
 
-        if not _is_on(mode, style, self._tv.powerstate):
-            mode = EFFECT_MODE
-            setting = None
-            if self._tv.powerstate in ("On", None):
-                style = "internal"
+        if effect.style == "OFF":
+            if self._last_selected_effect:
+                effect = self._last_selected_effect
             else:
-                style = "manual"
+                effect = AmbilightEffect(EFFECT_AUTO, "FOLLOW_VIDEO", "STANDARD")
+
+        if not effect.is_on(self._tv.powerstate):
+            effect.mode = EFFECT_MODE
+            effect.algorithm = None
+            if self._tv.powerstate in ("On", None):
+                effect.style = "internal"
+            else:
+                effect.style = "manual"
 
         if brightness is None:
             brightness = 255
 
         if hs_color is None:
-            hs_color = [0, 0]
+            hs_color = (0, 0)
 
-        if mode == EFFECT_MODE:
-            await self._set_ambilight_cached(style, hs_color, brightness)
-        elif mode == EFFECT_AUTO:
-            await self._set_ambilight_config(style, setting)
-        elif mode == EFFECT_EXPERT:
-            await self._set_ambilight_expert_config(
-                style, setting, hs_color, brightness
-            )
+        if effect.mode == EFFECT_MODE:
+            await self._set_ambilight_cached(effect, hs_color, brightness)
+        elif effect.mode == EFFECT_AUTO:
+            await self._set_ambilight_config(effect)
+        elif effect.mode == EFFECT_EXPERT:
+            await self._set_ambilight_expert_config(effect, hs_color, brightness)
 
         self._update_from_coordinator()
         self.async_write_ha_state()
@@ -378,7 +385,7 @@ class PhilipsTVLightEntity(CoordinatorEntity, LightEntity):
         if await self._tv.setAmbilightMode("internal") is False:
             raise Exception("Failed to set ambilight mode")
 
-        await self._set_ambilight_config("OFF", "")
+        await self._set_ambilight_config(AmbilightEffect(EFFECT_MODE, "OFF", ""))
 
         self._update_from_coordinator()
         self.async_write_ha_state()
