@@ -357,7 +357,9 @@ class SoundTouchDevice(MediaPlayerEntity):
     async def async_play_media(self, media_type, media_id, **kwargs):
         """Play a piece of media."""
         if media_source.is_media_source_id(media_id):
-            play_item = await media_source.async_resolve_media(self.hass, media_id)
+            play_item = await media_source.async_resolve_media(
+                self.hass, media_id, self.entity_id
+            )
             media_id = async_process_play_media_url(self.hass, play_item.url)
 
         await self.hass.async_add_executor_job(
